"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.findMatchingProjects = void 0;
const minimatch = require("minimatch");
const globCharacters = ['*', '|', '{', '}', '(', ')'];
const validPatternTypes = [
    'name',
    'tag', // Pattern is based on the project's tags
];
/**
 * Find matching project names given a list of potential project names or globs.
 *
 * @param patterns A list of project names or globs to match against.
 * @param projects A map of {@link ProjectGraphProjectNode} by project name.
 * @returns
 */
function findMatchingProjects(patterns = [], projects) {
    const projectNames = keys(projects);
    const patternObjects = patterns.map((p) => parseStringPattern(p, projects));
    const selectedProjects = new Set();
    const excludedProjects = new Set();
    for (const pattern of patternObjects) {
        // Handle wildcard with short-circuit, as its a common case with potentially
        // large project sets and we can avoid the more expensive glob matching.
        if (pattern.value === '*') {
            for (const projectName of projectNames) {
                if (pattern.exclude) {
                    excludedProjects.add(projectName);
                }
                else {
                    selectedProjects.add(projectName);
                }
            }
            continue;
        }
        if (pattern.type === 'tag') {
            for (const projectName of projectNames) {
                const tags = getItemInMapOrRecord(projects, projectName).data.tags || [];
                if (tags.includes(pattern.value)) {
                    (pattern.exclude ? excludedProjects : selectedProjects).add(projectName);
                    continue;
                }
                if (!globCharacters.some((c) => pattern.value.includes(c))) {
                    continue;
                }
                if (minimatch.match(tags, pattern.value).length)
                    (pattern.exclude ? excludedProjects : selectedProjects).add(projectName);
            }
            continue;
        }
        else if (pattern.type === 'name') {
            if (hasKey(projects, pattern.value)) {
                (pattern.exclude ? excludedProjects : selectedProjects).add(pattern.value);
                continue;
            }
            if (!globCharacters.some((c) => pattern.value.includes(c))) {
                continue;
            }
            const matchedProjectNames = minimatch.match(projectNames, pattern.value);
            for (const projectName of matchedProjectNames) {
                if (pattern.exclude) {
                    excludedProjects.add(projectName);
                }
                else {
                    selectedProjects.add(projectName);
                }
            }
        }
    }
    for (const project of excludedProjects) {
        selectedProjects.delete(project);
    }
    return Array.from(selectedProjects);
}
exports.findMatchingProjects = findMatchingProjects;
function keys(object) {
    return object instanceof Map ? [...object.keys()] : Object.keys(object);
}
function hasKey(object, key) {
    return object instanceof Map ? object.has(key) : key in object;
}
function getItemInMapOrRecord(object, key) {
    return object instanceof Map ? object.get(key) : object[key];
}
function parseStringPattern(pattern, projects) {
    let type;
    let value;
    const isExclude = pattern.startsWith('!');
    // Support for things like: `!{type}:value`
    if (isExclude) {
        pattern = pattern.substring(1);
    }
    const indexOfFirstPotentialSeparator = pattern.indexOf(':');
    if (indexOfFirstPotentialSeparator === -1 || hasKey(projects, pattern)) {
        type = 'name';
        value = pattern;
    }
    else {
        const potentialType = pattern.substring(0, indexOfFirstPotentialSeparator);
        if (isValidPatternType(potentialType)) {
            type = potentialType;
            value = pattern.substring(indexOfFirstPotentialSeparator + 1);
        }
        else {
            type = 'name';
            value = pattern;
        }
    }
    return { type, value, exclude: isExclude };
}
function isValidPatternType(type) {
    return validPatternTypes.includes(type);
}
//# sourceMappingURL=find-matching-projects.js.map